-- Best practices for parameterized queries

-- Use Parameters or Literals for Query Inputs:
-- If you use a local variable in a query predicate instead of a parameter or literal, the optimizer is unable to determine the value for the input.

-- estimates 9439.5 (1/3 of the table)
-- chooses merge join
declare @StartOrderDate datetime
set @StartOrderDate = '20040731'
select * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= @StartOrderDate


-- same query, cardinality 40
-- nested loop
SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= '20040731'

-- using sp_executesql appears to use the correct plan
EXEC sp_executesql
	N'SELECT * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
      WHERE h.SalesOrderID = d.SalesOrderId
      AND h.OrderDate >= @StartOrderDate',
	N'@StartOrderDate datetime',
	'20040731'

GO
CREATE PROCEDURE GetOrders(@StartOrderDate datetime)
as
--declare @StartOrderDate datetime
--set @StartOrderDate = '20040731'
select * from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= @StartOrderDate


execute GetOrders '20040731'
GO

-- Header only with L2E
CREATE PROCEDURE GetOrdersHeader(@StartOrderDate datetime)
as
select h.*, d.SalesOrderDetailID from Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
WHERE h.SalesOrderID = d.SalesOrderId
AND h.OrderDate >= @StartOrderDate


-- Even when local variables are used in a query, an estimate that is better than a guess is used in the case of equality predicates. Selectivity for conditions of the form "@local_variable = column_name" is estimated using the average value frequency from the histogram for column_name.

-- To eliminate the use of local variables, consider (1) rewriting the query to use literals instead of variables, (2) using sp_executesql with parameters that replace your use of local variables, or (3) using a stored procedure with parameters that replace your use of local variables. Dynamic SQL via EXEC may also be useful for eliminating local variables, but it typically will result in higher compilation overhead.

DROP PROCEDURE GetRecentSales
GO
CREATE PROCEDURE GetRecentSales (@date datetime) -- WITH RECOMPILE 
AS
BEGIN
	IF @date IS NULL
		SET @date = dateadd("mm",-3,(SELECT MAX(OrderDATE) 
							FROM Sales.SalesOrderHeader))
	SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
	WHERE h.SalesOrderID = d.SalesOrderID
	AND h.OrderDate > @date
END

EXECUTE GetRecentSales NULL

-- This SP, if called with NULL, will have the final SELECT statement optimized for @date = NULL. However, at run time, the date is not NULL, but three months before the latest OrderDate.  The actual number of SalesOrderHeader rows that qualify is 5,736. The optimizer chooses a nested loop join for the query when NULL is passed to GetRecentSales, whereas the optimal plan contains a merge join.

CREATE PROCEDURE GetRecentSales (@date datetime) WITH RECOMPILE AS
BEGIN
	IF @date IS NULL
		SET @date = dateadd("mm",-3,(SELECT MAX(OrderDATE) 
							FROM Sales.SalesOrderHeader))
	SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
	WHERE h.SalesOrderID = d.SalesOrderID
	AND h.OrderDate > @date
END
go

SET STATISTICS PROFILE ON
GO

execute GetRecentSales NULL WITH RECOMPILE
EXECUTE GetRecentSales '20040430' WITH RECOMPILE

EXECUTE GetRecentSales '20060810' WITH RECOMPILE
go

SET STATISTICS PROFILE OFF
GO

CREATE PROCEDURE GetRecentSales2 (@date datetime)  AS
BEGIN
	IF @date IS NULL
		SET @date = dateadd("mm",-3,(SELECT MAX(OrderDATE) 		
                FROM Sales.SalesOrderHeader))
	EXEC GetRecentSalesHelper @date
END
go
DROP PROCEDURE GetRecentSalesHelper
CREATE PROCEDURE GetRecentSalesHelper (@date datetime) -- WITH RECOMPILE 
AS
BEGIN
	SELECT * FROM Sales.SalesOrderHeader h, Sales.SalesOrderDetail d
	WHERE h.SalesOrderID = d.SalesOrderID
	AND h.OrderDate > @date -- @date is unchanged from compile time,
					-- so a good plan is obtained.
END
go

execute GetRecentSales NULL
execute GetRecentSales2 NULL